#!/bin/bash
set -e
set -o pipefail

# Logger methods ##########################################
###########################################################
log_proc_marker() {
  local prompt="$@"
  echo ""
  echo "# $(date +"%T") #######################################"
  echo "# $prompt"
  echo "##################################################"
}

log_proc_msg() {
  local message="$@"
  echo "|___ $@"
}

log_proc_error() {
  local message="$1"
  local error="$2"
  local bold_red_text='\e[91m'
  local reset_text='\033[0m'

  echo -e "$bold_red_text|___ $message$reset_text"
  echo -e "     $error"
}

log_proc_success() {
  local message="$@"
  local bold_green_text='\e[32m'
  local reset_text='\033[0m'

  echo -e "$bold_green_text|___ $message$reset_text"
}

# End Logger methods ######################################
###########################################################

export LOGS_FILE="$RUNTIME_DIR/logs/$SERVICE_NAME.log"
export KUBECONFIG="$CONFIG_DIR/kubernetes.conf"
export SERVICE_DEPLOY_CONFIG="$CONFIG_DIR/master/$SERVICE_NAME"_deployment.yml

## Write logs of this script to component specific file
exec &> >(tee -a "$LOGS_FILE")

__validate_service_configs() {
  log_proc_msg "Service $SERVICE_NAME configuration"
  log_proc_msg "SERVICE: $SERVICE_NAME"
  log_proc_msg "LOGS_FILE: $LOGS_FILE"
  log_proc_msg "SCRIPTS_DIR: $SCRIPTS_DIR"
  log_proc_msg "RUN_MODE: $RUN_MODE"
}

__cleanup_containers() {
  log_proc_msg "Stopping stale container for the service"
  docker rm -f $SERVICE_NAME || true
}

__run_service() {
  log_proc_msg "Running service: $SERVICE_NAME"
  local run_output=$($RUN_COMMAND)
  if [ "$?" -ne 0 ] || [[ "$run_output" == *"Error"* ]]; then
    log_proc_error "Failed to start service"
    log_proc_error "Docker run returned: $run_output"
    exit 1
  else
    log_proc_msg "Successfully started service: $run_output"
  fi
}

__cleanup_deployments() {
  log_proc_msg "Removing stale service deployments"
  if [ ! -f "$KUBECONFIG" ]; then
    log_proc_error "No kubeconfig file present, exiting"
    exit 1
  fi

  local service_deployment=$(echo "$SERVICE_NAME" | tr '[:upper:]' '[:lower:]' )-deployment
  kubectl --kubeconfig $KUBECONFIG \
    delete deployment \
    "$service_deployment" || true
}

__cleanup_configmap() {
  log_proc_msg "Deleting configmap for service: $SERVICE_NAME"

  if [ ! -f "$KUBECONFIG" ]; then
    log_proc_error "No kubeconfig file present, exiting"
    exit 1
  fi

  local config_map_name=$(echo "$SERVICE_NAME" | tr '[:upper:]' '[:lower:]' )-config
  kubectl --kubeconfig $KUBECONFIG \
    delete configmap \
    "$config_map_name" || true
}

__create_deployment() {
  log_proc_msg "Creating deployment: $SERVICE_NAME"

  if [ ! -f "$KUBECONFIG" ]; then
    log_proc_error "No kubeconfig file present, exiting"
    exit 1
  fi

  local config_map_name=$(echo "$SERVICE_NAME" | tr '[:upper:]' '[:lower:]' )-config
  local config_map_path="$CONFIG_DIR/master/$SERVICE_NAME".env

  log_proc_msg "Creating config map for service: $SERVICE_NAME"
  if [ ! -f "$config_map_path" ]; then
    log_proc_error "No config map present for $SERVICE_NAME"
    exit 1
  fi

  kubectl --kubeconfig $KUBECONFIG \
    create configmap \
    "$config_map_name" \
    --from-env-file="$config_map_path"

  if [ ! -f "$SERVICE_DEPLOY_CONFIG" ]; then
    log_proc_error "Deploy config missing for service: $SERVICE_DEPLOY_CONFIG"
    exit 1
  else
    log_proc_msg "Deploy config found, creating a new deployment"
    kubectl --kubeconfig $KUBECONFIG \
      apply -f "$SERVICE_DEPLOY_CONFIG"

    ## wait for kube api to process this message before
    ## creating other components
    sleep 3
  fi
}

main() {
  if [ -z "$SERVICE_NAME" ] || [ "$SERVICE_NAME" == "" ]; then
    log_proc_error "'SERVICE_NAME' env not present, exiting"
    exit 1
  else
    log_proc_marker "Booting service: $SERVICE_NAME"
    __validate_service_configs
    if [ "$RUN_MODE" == "devmode" ] || [ "$RUN_MODE" == "onebox" ]; then
      __cleanup_containers
      __run_service
    else
      __cleanup_deployments
      __cleanup_configmap
      __create_deployment
    fi
  fi
}


main
