#!/bin/bash
set -e
set -o pipefail

__clean_dirs() {
  log_proc_msg "Cleaning up directories"
  if [ -d "$CONFIG_DIR" ]; then
    log_proc_msg "Removing previously created $CONFIG_DIR"
    rm -rf "$CONFIG_DIR"
  fi
  mkdir -p "$CONFIG_DIR"

  if [ -d "$RUNTIME_DIR" ]; then
    log_proc_msg "Removing previously created $RUNTIME_DIR"
    rm -rf "$RUNTIME_DIR"
  fi
  mkdir -p "$RUNTIME_DIR"
}

__remove_services() {
  log_proc_msg "Removing services"
  local service_count=$(docker service ls -q | wc -l)
  if [[ "$service_count" -ne 0 ]]; then
    local service_ids=$(docker service ls -q)
    local remove_service_cmd="docker service rm"
    log_proc_msg "Executing: $remove_service_cmd"
    for id in ${service_ids[@]}; do
      eval "$remove_service_cmd $id"
    done
  else
    log_proc_msg "No services found. Skipping removal."
  fi

  local count=$(docker ps -aq | wc -l)
  if [[ "$count" -ne 0 ]]; then
    local container_ids=$(docker ps -aq)
    local remove_cmd="docker rm -f"
    log_proc_msg "Executing: $remove_cmd"
    for id in ${container_ids[@]}; do
      eval "$remove_cmd $id"
    done
  else
    log_proc_msg "No containers found. Skipping removal."
  fi
}

__remove_kubernetes_services() {
  log_proc_msg "Removing kubernetes services"

  local services=("db" "vault" "msg" "redis" "api" "www")
  for service in ${services[@]}; do
    log_proc_msg "Removing service: $service"
    kubectl --kubeconfig $KUBECONFIG delete service $service
  done
}

__remove_deployments() {
  log_proc_msg "Removing kubernetes services"

  local deployments=("vault-deployment" \
    "msg-deployment" \
    "redis-deployment" \
    "api-deployment" \
    "www-deployment" \
    "steptrigger-deployment" \
    "pipelinesync-deployment" \
    "runtrigger-deployment" \
    "cron-deployment" \
    "nexec-deployment")

  for deployment in ${deployments[@]}; do
    log_proc_msg "Removing deployment: $deployment"
    kubectl --kubeconfig $KUBECONFIG delete deployment $deployment
  done
}

__drop_database() {
  log_proc_msg "Dropping database"
  source "$INSTALLER_SCRIPTS_DIR/dropDatabase.sh"
}

__stop_installer() {
  log_proc_msg "Stopping installer container"
  sudo docker rm -f installer || true
}

__cleanup_proxy() {
  log_proc_msg "Cleaning up proxy settings from $SYSTEM_ENV"

  if [ ! -w "$SYSTEM_ENV" ]; then
    log_proc_warn "Skipping proxy cleanup"
    log_proc_warn "User $INSTALL_USER does not have permissions to edit $SYSTEM_ENV"
    return
  fi

  log_proc_msg "Removing http_proxy"
  unset http_proxy
  sed -i '/^export http_proxy=/d' $SYSTEM_ENV || true

  log_proc_msg "Removing https_proxy"
  unset https_proxy
  sed -i '/^export https_proxy=/d' $SYSTEM_ENV || true

  log_proc_msg "Removing no_proxy"
  unset no_proxy
  sed -i '/^export no_proxy=/d' $SYSTEM_ENV || true
}

main() {
  log_proc_marker "Cleaning installation"
  log_proc_msg "WARNING: this will erase your Pipelines installation..."
  log_proc_msg "WARNING: including all containers and the database."
  log_proc_msg "Do you wish to continue? (Y to confirm)"
  read -r confirmation
  confirmation=$(echo "$confirmation" | awk '{print toupper($0)}')

  if [ ! -f "$PIPELINES_CONFIG" ]; then
    log_proc_error "Installer config file not found, skipping cleanup"
    exit 1
  fi

  source "$PIPELINES_CONFIG"
  if [[ "$confirmation" =~ "Y" ]]; then
    if [ "$RUN_MODE" == "devmode" ] || [ "$RUN_MODE" == "onebox" ]; then
      log_proc_msg "Removing all containers"
      __remove_services
      __cleanup_proxy
      __clean_dirs
    else
      __stop_installer
      __remove_kubernetes_services
      __remove_deployments
      ## all services that access db need to be stopped before dropping db
      __drop_database
      __cleanup_proxy
      __clean_dirs
    fi
    log_proc_success "Clean is complete. Reinstall by running './pipelines install'"
    exit 0
  else
    log_proc_error "Cancelling clean"
    exit 1
  fi
}

main
