#!/bin/bash
set -e
set -o pipefail

# Logger methods ##########################################
###########################################################
log_proc_marker() {
  local prompt="$@"
  echo ""
  echo "# $(date +"%T") #######################################"
  echo "# $prompt"
  echo "##################################################"
}

log_proc_msg() {
  local message="$@"
  echo "|___ $@"
}

log_proc_error() {
  local message="$1"
  local error="$2"
  local bold_red_text='\e[91m'
  local reset_text='\033[0m'

  echo -e "$bold_red_text|___ $message$reset_text"
  echo -e "     $error"
}

log_proc_success() {
  local message="$@"
  local bold_green_text='\e[32m'
  local reset_text='\033[0m'

  echo -e "$bold_green_text|___ $message$reset_text"
}

# End Logger methods ######################################
###########################################################

export KUBECONFIG="$CONFIG_DIR/kubernetes.conf"
export LOGS_FILE="$RUNTIME_DIR/logs/$SERVICE_NAME.log"

## Write logs of this script to component specific file
exec &> >(tee -a "$LOGS_FILE")

__validate_service_configs() {
  log_proc_msg "Service $SERVICE_NAME configuration"
  log_proc_msg "SERVICE: $SERVICE_NAME"
  log_proc_msg "RUN_MODE: $RUN_MODE"
  log_proc_msg "LOGS_FILE: $LOGS_FILE"
}

__cleanup_containers() {
  log_proc_msg "Removing container for the service: $SERVICE_NAME"
  sudo docker rm -f $SERVICE_NAME || true
}

__cleanup_configmap() {
  log_proc_msg "Deleting configmap for service: $SERVICE_NAME"

  if [ ! -f "$KUBECONFIG" ]; then
    log_proc_error "No kubeconfig file present, exiting"
    exit 1
  fi

  local config_map_name=$(echo "$SERVICE_NAME" | tr '[:upper:]' '[:lower:]' )-config
  kubectl --kubeconfig $KUBECONFIG \
    delete configmap \
    "$config_map_name" || true
}

__cleanup_deployments() {
  log_proc_msg "Removing stale service deployments"
  if [ ! -f "$KUBECONFIG" ]; then
    log_proc_error "No kubeconfig file present, exiting"
    exit 1
  fi

  local service_deployment="$SERVICE_NAME"-deployment
  kubectl --kubeconfig $KUBECONFIG \
    delete deployment \
    "$service_deployment" || true
}

main() {
  if [ -z "$SERVICE_NAME" ] || [ "$SERVICE_NAME" == "" ]; then
    log_proc_error "'SERVICE_NAME' env not present, exiting"
    exit 1
  elif [ "$RUN_MODE" == "devmode" ] || [ "$RUN_MODE" == "onebox" ]; then
    log_proc_marker "Removing service: $SERVICE_NAME"
    __validate_service_configs
    __cleanup_containers

  else
    log_proc_marker "Removing service: $SERVICE_NAME"
    __validate_service_configs
    __cleanup_configmap
    __cleanup_deployments
  fi
}

main
