#!/bin/bash
set -e
set -o pipefail

export COMPONENT="vault"
export VAULT_DATA_DIR="$RUNTIME_DIR/$COMPONENT/data"
export VAULT_CONFIG_DIR="$CONFIG_DIR/$COMPONENT"
export LOGS_FILE="$RUNTIME_DIR/logs/$COMPONENT.log"
export VAULT_INSTALL_SCRIPT="installVault.sh"
export VAULT_CONFIG="$VAULT_CONFIG_DIR/vault_config.hcl"
export VAULT_IMAGE="$IMAGE_REGISTRY_URL/$VAULT_IMAGE:$RELEASE"

## Write logs of this script to component specific file
exec &> >(tee -a "$LOGS_FILE")

__validate_vault_envs() {
  log_proc_msg "Initializing vault environment variables"
  log_proc_msg "VAULT_DATA_DIR: $VAULT_DATA_DIR"
  log_proc_msg "VAULT_CONFIG_DIR: $VAULT_CONFIG_DIR"
  log_proc_msg "VAULT_IP: $VAULT_IP"
  log_proc_msg "VAULT_PORT: $VAULT_PORT"
  log_proc_msg "LOGS_FILE: $LOGS_FILE"
}

__validate_vault_mounts() {
  log_proc_msg "Validating vault mounts"
  if [ ! -d "$VAULT_DATA_DIR" ]; then
    log_proc_msg "Creating data directory $VAULT_DATA_DIR"
    mkdir -p $VAULT_DATA_DIR
  else
    log_proc_msg "Data directory already present: $VAULT_DATA_DIR"
  fi

  if [ ! -d "$VAULT_CONFIG_DIR" ]; then
    log_proc_msg "Creating config directory $VAULT_CONFIG_DIR"
    mkdir -p $VAULT_CONFIG_DIR
  else
    log_proc_msg "Config directory already present: $VAULT_CONFIG_DIR"
  fi

  mkdir -p $VAULT_CONFIG_DIR/scripts
}

__update_vault_creds() {
  log_proc_msg "Updating vault to database connection credentials"

  local vault_config_template="$INSTALLER_SCRIPTS_DIR/configs/vault_config.hcl.template"
  cp -vr $vault_config_template $VAULT_CONFIG

  log_proc_msg "Updating db host"
  sed -i 's#{{DB_IP}}#'$DB_IP'#g' $VAULT_CONFIG

  log_proc_msg "Updating db port"
  sed -i 's#{{DB_PORT}}#'$DB_PORT'#g' $VAULT_CONFIG

  log_proc_msg "Updating vault port"
  sed -i 's#{{VAULT_PORT}}#'$VAULT_PORT'#g' $VAULT_CONFIG

  log_proc_msg "Updating db name"
  sed -i 's#{{DB_NAME}}#'$DB_NAME'#g' $VAULT_CONFIG

  log_proc_msg "Updating db password"
  sed -i 's#{{DB_PASSWORD}}#'$DB_PASSWORD'#g' $VAULT_CONFIG

  log_proc_msg "Updating db username"
  sed -i 's#{{DB_USER}}#'$DB_USER'#g' $VAULT_CONFIG
}

__install_vault_compose() {
    source "$INSTALLER_SCRIPTS_DIR/compose/$VAULT_INSTALL_SCRIPT"
}

__install_vault_kubernetes() {
    source "$INSTALLER_SCRIPTS_DIR/kubernetes/$VAULT_INSTALL_SCRIPT"
}


main() {
  log_proc_marker "Installing Vault"
  if [ "$RUN_MODE" == "devmode" ] || [ "$RUN_MODE" == "onebox" ]; then
    __validate_vault_envs
    __validate_vault_mounts
    __update_vault_creds
    __install_vault_compose
  else
    log_proc_msg "Vault not installed"
    __validate_vault_envs
    __validate_vault_mounts
    __update_vault_creds
    __install_vault_kubernetes
  fi
  log_proc_msg "Vault installed successfully"
}

main
