#!/bin/bash
set -e
set -o pipefail

export REDIS_DEPLOYMENT="redis-deployment"
export REDIS_DEPLOYMENT_CONFIG="$REDIS_CONFIG_DIR/redis_deployment.yml"
export REDIS_SERVICE_CONFIG="$REDIS_CONFIG_DIR/redis_service.yml"

__update_redis_ip() {
  log_proc_msg "Updating redis service IP"
  local existing_service=$(kubectl \
    --kubeconfig=$KUBECONFIG \
    get services -o json \
    | jq '.items[] | select(.metadata.name == "redis")')
  if [ -z "$existing_service" ]; then
    log_proc_error "redis service not created in k8s cluster, exiting"
    exit 1
  else
    log_proc_msg "Found redis service, updating IP address"
    local redis_ip=$(echo "$existing_service" \
      | jq -r '.spec.clusterIP')
    set_config "REDIS_IP" "$redis_ip"
  fi
}

__create_service() {
  log_proc_msg "Creating service for Redis"

  local existing_service=$(kubectl \
    --kubeconfig=$KUBECONFIG \
    get services -o json \
    | jq '.items[] | select(.metadata.name == "redis")')
  if [ -z "$existing_service" ]; then
    local service_template="$INSTALLER_SCRIPTS_DIR/configs/redis_service.yml.template"

    cp -vr "$service_template" "$REDIS_SERVICE_CONFIG"
    sed -i "s#{{REDIS_PORT}}#$REDIS_PORT#g" $REDIS_SERVICE_CONFIG
    kubectl \
      --kubeconfig=$KUBECONFIG \
      create -f "$REDIS_SERVICE_CONFIG"
  else
    log_proc_msg "Service already exists for redis, skipping"
  fi
}

__cleanup_service() {
  log_proc_msg "Cleaning up configmap for redis"

  kubectl --kubeconfig $KUBECONFIG \
    delete service redis || true
}

__create_deployment() {
  log_proc_msg "Creating deployment"

  if [ ! -f "$REDIS_DEPLOYMENT_CONFIG" ]; then
    log_proc_error "Deploy config missing for redis: $REDIS_DEPLOYMENT_CONFIG"
    exit 1
  else
    log_proc_msg "Deploy config found, creating a new deployment"
    kubectl --kubeconfig $KUBECONFIG \
      apply -f "$REDIS_DEPLOYMENT_CONFIG"
  fi
}

__cleanup_deployment() {
  log_proc_msg "Cleaning up deployment"

  kubectl --kubeconfig $KUBECONFIG \
    delete deployments "$REDIS_DEPLOYMENT" || true
}

__update_deployment_config() {

  local redis_replicas=1
  local deployment_template="$INSTALLER_SCRIPTS_DIR/configs/redis_deployment.yml.template"
  cp -vr $deployment_template $REDIS_DEPLOYMENT_CONFIG

  log_proc_msg "Updating replicas"
  sed -i 's#{{DEPLOYMENT_NAME}}#'$REDIS_DEPLOYMENT'#g' $REDIS_DEPLOYMENT_CONFIG

  log_proc_msg "Updating image registry secret"
  sed -i 's#{{IMAGE_REGISTRY_SECRET}}#'$IMAGE_REGISTRY_SECRET'#g' $REDIS_DEPLOYMENT_CONFIG

  log_proc_msg "Updating replicas"
  sed -i 's#{{REPLICAS}}#'$redis_replicas'#g' $REDIS_DEPLOYMENT_CONFIG

  log_proc_msg "Updating image name"
  sed -i 's#{{REDIS_IMAGE}}#'$REDIS_IMAGE'#g' $REDIS_DEPLOYMENT_CONFIG

  log_proc_msg "Updating port"
  sed -i 's#{{REDIS_PORT}}#'$REDIS_PORT'#g' $REDIS_DEPLOYMENT_CONFIG

  log_proc_msg "Updating volume mount"
  sed -i 's#{{REDIS_DATA_DIR}}#'$REDIS_DATA_DIR'#g' $REDIS_DEPLOYMENT_CONFIG

  log_proc_msg "Updating volume mount"
  sed -i 's#{{REDIS_CONFIG_DIR}}#'$REDIS_CONFIG_DIR'#g' $REDIS_DEPLOYMENT_CONFIG
}

__check_redis() {
  log_proc_msg "Checking redis container status on: $REDIS_IP:$REDIS_PORT"
  platform_exec "check_service_connection" "$REDIS_IP" "$REDIS_PORT" "$COMPONENT"
}

main() {
  __update_deployment_config
  __cleanup_deployment
  __create_deployment
  __cleanup_service
  __create_service
  __update_redis_ip
  __check_redis
}

main
