#!/bin/bash
set -e

__log_status_marker() {
  if [ "$format" != "json" ]; then
    log_proc_marker "$1"
  fi
}

__parse_response() {
  local api_response="$1"
  local api_response_status_code="$2"
  local current_object="$3"

  if [ $api_response_status_code -gt 299 ]; then
    if [ "$format" == "json" ]; then
      local response_object=$(jq --arg "error" "$api_response" \
        ". | .error=\$error" <<<'{}')
      jq_args+=( --argjson "$current_object" "$response_object" )
      jq_query+=" | .$current_object=\$$current_object"
      jq "${jq_args[@]}" "$jq_query" <<<'{}'
    else
      log_proc_error "Response status code: $api_response_status_code"
      log_proc_error "Response: $api_response"
    fi
    exit 1
  fi

  if [ -z "$api_response" ]; then
    if [ "$format" == "json" ]; then
      local response_object=$(jq --arg "error" \
        "No response received from api" ". | .error=\$error" <<<'{}')
      jq_args+=( --argjson "$current_object" "$response_object" )
      jq_query+=" | .$current_object=\$$current_object"
      jq "${jq_args[@]}" "$jq_query" <<<'{}'
    else
      log_proc_error "No response received from api, exiting"
    fi
    exit 1
  fi

  local response=$(echo $api_response | jq '.' 2>&1)
  if [[ $response = *"parse error"* ]]; then
    if [ "$format" == "json" ]; then
      local response_object=$(jq --arg "error" \
        "Invalid response from api: $api_response" \
        ". | .error=\$error" <<<'{}')
      jq_args+=( --argjson "$current_object" "$response_object" )
      jq_query+=" | .$current_object=\$$current_object"
      jq "${jq_args[@]}" "$jq_query" <<<'{}'
    else
      log_proc_error "Invalid response from api: $api_response"
    fi
    exit 1
  fi
}

__print_status() {
  local status_component="$1"
  local status_data="$2"
  local component_ip=$(echo $status_data | jq -r '.ip')
  local component_port=$(echo $status_data | jq -r '.port')
  local component_uptime=$(echo $status_data | jq -r '.uptime')
  local component_reachable=$(echo $status_data | jq -r '.isReachable')

  printf "%2s COMPONENT %5s IP %9s PORT %10s UPTIME %8s REACHABLE %5s Errors\n"
  printf "%10s %15s %10s %20s %10s \n" "$status_component" "$component_ip" \
    "$component_port" "$component_uptime" "$component_reachable"

  printf "\n"
}

__print_master_status() {
  local status_data="$1"
  local component_clusterName=$(echo $status_data | jq -r '.clusterName')
  local component_masterIP=$(echo $status_data | jq -r '.masterIP')
  local component_reachable=$(echo $status_data | jq -r '.isReachable')
  local component_errors=$(echo $status_data | jq -r '.error')

  printf "CLUSTER NAME %3s $component_clusterName\n"
  printf "MASTER %9s $component_masterIP\n"
  printf "REACHABLE %6s $component_reachable\n"

  printf "WORKERS"
  local workers=false
  echo $status_data | jq -r '.workers[]' | while read worker; do
    if [ "$workers" == "false" ]; then
      workers=true
      printf " %8s $worker"
    else
      printf "\n%16s $worker"
    fi
  done
  printf "\n"

  if [ "$component_errors" == "null" ]; then
    printf "Errors\n"
  else
    printf "Errors %9s $component_errors\n"
  fi

  printf "\n"
}

__get_installer_status() {
  __log_status_marker "Checking installer status"
  _get_installer_status
  __parse_response "$response" "$response_status_code" "installer"
  if [ "$format" == "json" ]; then
    jq_args+=( --argjson "installer" "$response" )
    jq_query+=" | .installer=\$installer"
  else
    __print_status "installer" "$response"
  fi
}

__get_db_status() {
  __log_status_marker "Checking database status"
  _get_db_status
  __parse_response "$response" "$response_status_code" "db"
  if [ "$format" == "json" ]; then
    jq_args+=( --argjson "db" "$response" )
    jq_query+=" | .db=\$db"
  else
    __print_status "db" "$response"
  fi
}

__get_cluster_status() {
  __log_status_marker "Checking cluster status"
  _get_master_status
  __parse_response "$response" "$response_status_code" "master"
  if [ "$format" == "json" ]; then
    jq_args+=( --argjson "master" "$response" )
    jq_query+=" | .master=\$master"
  else
    __print_master_status "$response"
  fi
}

__get_uptime() {
  __log_status_marker "Local machine uptime"

  if [ "$format" == "json" ]; then
    local uptime=$(uptime -p)
    jq_args+=( --arg "localUptime" "$uptime" )
    jq_query+=" | .localUptime=\$localUptime"
  else
    uptime -p
  fi
}

__log_system_information() {
  if [ "$format" == "json" ]; then
    local systemInfo=$(log_system_info "json")
    jq_args+=( --argjson "system" "$systemInfo" )
    jq_query+=" | .system=\$system"
  else
    log_system_info
  fi
}

__print_configured_replicas() {
  __log_status_marker "Configured replicas"
  if [ "$format" == "json" ]; then
    local replicas_args=( )
    local replicas_query='.'

    replicas_args+=( --argjson "api" "$API_REPLICA_COUNT" )
    replicas_query+=" | .api=\$api"

    replicas_args+=( --argjson "www" "$WWW_REPLICA_COUNT" )
    replicas_query+=" | .www=\$www"

    replicas_args+=( --argjson "pipelinesync" "$PIPELINESYNC_REPLICA_COUNT" )
    replicas_query+=" | .pipelinesync=\$pipelinesync"

    replicas_args+=( --argjson "runtrigger" "$RUNTRIGGER_REPLICA_COUNT" )
    replicas_query+=" | .runtrigger=\$runtrigger"

    replicas_args+=( --argjson "steptrigger" "$STEPTRIGGER_REPLICA_COUNT" )
    replicas_query+=" | .steptrigger=\$steptrigger"

    replicas_args+=( --argjson "cron" "$CRON_REPLICA_COUNT" )
    replicas_query+=" | .cron=\$cron"

    replicas_args+=( --argjson "nexec" "$NEXEC_REPLICA_COUNT" )
    replicas_query+=" | .nexec=\$nexec"

    local configuredReplicas=$(jq "${replicas_args[@]}" \
      "$replicas_query" <<<'{}')

    jq_args+=( --argjson "configuredReplicas" "$configuredReplicas" )
    jq_query+=" | .configuredReplicas=\$configuredReplicas"
  else
    printf "api %12s $API_REPLICA_COUNT\n"
    printf "www %12s $WWW_REPLICA_COUNT\n"
    printf "pipelinesync %3s $PIPELINESYNC_REPLICA_COUNT\n"
    printf "runtrigger %5s $RUNTRIGGER_REPLICA_COUNT\n"
    printf "steptrigger %4s $STEPTRIGGER_REPLICA_COUNT\n"
    printf "cron %11s $CRON_REPLICA_COUNT\n"
    printf "nexec %10s $NEXEC_REPLICA_COUNT\n"
    printf "\n"
  fi
}

main() {
  source "$PIPELINES_CONFIG"
  local format="$1"
  if [ "$format" == "json" ]; then
    local jq_args=( )
    local jq_query='.'
  fi

  __get_installer_status
  __get_db_status
  __get_cluster_status
  __get_uptime
  __log_system_information
  __print_configured_replicas

  if [ "$format" == "json" ]; then
    jq "${jq_args[@]}" "$jq_query" <<<'{}'
  fi
}

main "$@"
