#!/bin/bash
set -e
set -o pipefail

export UPGRADE_VERSION=""
readonly UPDATE_TIME="$(date)"

__load_config() {
  log_proc_msg "Loading system configuration"
  source "$PIPELINES_CONFIG"
  log_system_info
}

__archive_files() {
  log_proc_marker "Backing up config and logs"
  mkdir -p $TMP_DIR/backup
  cp -r $LOGS_DIR $TMP_DIR/backup/logs
  if [ -f $PIPELINES_CONFIG ]; then
    cp $PIPELINES_CONFIG $TMP_DIR/backup/pipelines.env
  fi
  if [ -f $REGISTRY_CREDS ]; then
    cp $REGISTRY_CREDS $TMP_DIR/backup/registry_creds.json
  fi
  if [ -f $KUBECONFIG ]; then
    cp $KUBECONFIG $TMP_DIR/backup/kubernetes.conf
  fi
  if [ -f $SYSTEM_ENV ]; then
    cp $SYSTEM_ENV $TMP_DIR/backup/environment
  fi

  local backup_directory=$RUNTIME_DIR/archives
  local archive_file="$backup_directory/$TIMESTAMP.tar.gz"

  mkdir -p $backup_directory
  tar -czf $archive_file -C $TMP_DIR/backup .

  local config_backup_count=$CONFIG_BACKUP_COUNT

  if [ -z "$config_backup_count" ]; then
    config_backup_count=5
  fi

  ls -t $backup_directory | awk "NR>$config_backup_count" | \
    awk -v dir=$backup_directory '{printf("%s/%s\n", dir, $1); }' | \
    xargs rm -f

  rm -r $TMP_DIR/backup

  log_proc_success "Saved previous configuration in $archive_file"
}

__set_upgrade_version() {
  log_proc_marker "Setting upgrade version"
  local version_file="$INSTALLER_HOME/version.txt"
  if [ -f  "$version_file" ]; then
    local upgrade_version=$(cat "$INSTALLER_HOME/version.txt")
    if [ ! -z "$upgrade_version" ]; then
      UPGRADE_VERSION="$upgrade_version"
    else
      log_proc_error "Unknown upgrade version"
      exit 1
    fi
  else
    log_proc_error "Missing upgrade version file: $version_file"
    exit 1
  fi
  log_proc_success "Upgrading from $RELEASE to $UPGRADE_VERSION"
}

__create_secrets() {
  log_proc_msg "Creating secrets to be used by deployments"
  source "$INSTALLER_SCRIPTS_DIR/createSecrets.sh"
}

__pull_images() {
  #TODO: in cluster mode, pull images on master and workers
  log_proc_marker "Pulling images for tag: $UPGRADE_VERSION"
  platform_exec "pull_service_images" "$UPGRADE_VERSION"
}

__upgrade_installer() {
  log_proc_msg "Booting installer container"
  # TODO: either boot container or start service based on config
  platform_exec "boot_installer" "$UPGRADE_VERSION"
  check_installer_connection
  # let installer initialize all routes
  sleep 10
}

__stop_stateless_services() {
  log_proc_marker "Stopping stateless services"

  local services=("pipelineSync" "runTrigger" "stepTrigger" "cron" "nexec")
  for service in ${services[@]}; do
    log_proc_msg "Stopping: $service"
    _delete_service "$service"
    if [ $is_request_success == true ]; then
      log_proc_success "$service deleted"
    else
      log_proc_error "Error while deleting $service"
      log_proc_error "Status code: $response_status_code"
      log_proc_error "==================== Error Logs ====================="
      local logs_file="$LOGS_DIR/$service.log"
      tail -$MAX_ERROR_LOG_COUNT $logs_file
      log_proc_error "====================================================="
      exit 1
    fi
  done
}

__stop_stateful_services() {
  local services=("api" "www")
  for service in ${services[@]}; do
    log_proc_msg "Stopping: $service"
    _delete_service "$service"
    if [ $is_request_success == true ]; then
      log_proc_success "$service deleted"
    else
      log_proc_error "Error while deleting $service"
      log_proc_error "Status code: $response_status_code"
      log_proc_error "==================== Error Logs ====================="
      local logs_file="$LOGS_DIR/$service.log"
      tail -$MAX_ERROR_LOG_COUNT $logs_file
      log_proc_error "====================================================="
      exit 1
    fi
  done
}

__run_migrations() {
  log_proc_msg "Running migrations"
  source "$INSTALLER_SCRIPTS_DIR/initializeDb.sh"
}

__boot_api() {
  log_proc_msg "Booting API"
  local service='{"name": "api"}'
  _post_services "$service"
  if [ $is_request_success == true ]; then
    sleep 6
    log_proc_success "API started"
  else
    log_proc_error "Error while starting API"
    log_proc_error "Status code: $response_status_code"
    exit 1
  fi
}

__boot_www() {
  log_proc_msg "Booting WWW"
  local service='{"name": "www"}'
  _post_services "$service"
  if [ $is_request_success == true ]; then
    log_proc_success "WWW started"
  else
    log_proc_error "Error while starting WWW "
    log_proc_error "Status code: $response_status_code"
    exit 1
  fi
}

__boot_services() {
  log_proc_msg "Booting services"
  local services=("pipelineSync" "runTrigger" "stepTrigger" "cron" "nexec")
  for service in ${services[@]}; do
    local service_data='{"name": "'$service'"}'

    _post_services "$service_data"
    if [ $is_request_success == true ]; then
      log_proc_success "$service started"
    else
      log_proc_error "Error while starting $service"
      log_proc_error "Status code: $response_status_code"
      exit 1
    fi
  done
}

__update_release_version() {
  log_proc_marker "All upgrade steps completed, updating release version in config to: $UPGRADE_VERSION"
  set_config "RELEASE" "$UPGRADE_VERSION"
  set_config "LAST_UPDATE_TIME" "$UPDATE_TIME"
}

main() {
  log_proc_marker "Upgrading Pipelines"
  __load_config
  __archive_files
  __set_upgrade_version
  __create_secrets
  __pull_images
  __upgrade_installer
  __stop_stateless_services
  __run_migrations
  __stop_stateful_services
  __boot_api
  __boot_www
  __boot_services
  __update_release_version

}

{
  main
} 2>&1 | tee "$LOG_FILE" ; ( exit ${PIPESTATUS[0]} )
