#!/bin/bash
set -e
set -o pipefail

# Logger methods ##########################################
###########################################################
log_proc_marker() {
  local prompt="$@"
  echo ""
  echo "# $(date +"%T") #######################################"
  echo "# $prompt"
  echo "##################################################"
}

log_proc_msg() {
  local message="$@"
  echo "|___ $@"
}

log_proc_error() {
  local message="$1"
  local error="$2"
  local bold_red_text='\e[91m'
  local reset_text='\033[0m'

  echo -e "$bold_red_text|___ $message$reset_text"
  echo -e "     $error"
}

log_proc_success() {
  local message="$@"
  local bold_green_text='\e[32m'
  local reset_text='\033[0m'

  echo -e "$bold_green_text|___ $message$reset_text"
}

# End Logger methods ######################################
###########################################################

export COMPONENT="db"
export LOGS_FILE="$RUNTIME_DIR/logs/$COMPONENT.log"
export FAKE_API_CONTAINER_NAME="fakeapi"

## Write logs of this script to component specific file
exec &> >(tee -a "$LOGS_FILE")

__validate_api_envs() {
  log_proc_msg "Initializing api environment variables"
  log_proc_msg "CONFIG_DIR: $CONFIG_DIR"
  log_proc_msg "RELEASE: $RELEASE"
  log_proc_msg "API_IMAGE: $API_IMAGE"
  log_proc_msg "DBNAME: $DBNAME"
  log_proc_msg "DBUSERNAME: $DBUSERNAME"
  log_proc_msg "DBPASSWORD: ${#DBPASSWORD}"
  log_proc_msg "DBHOST: $DBHOST"
  log_proc_msg "DBPORT: $DBPORT"
  log_proc_msg "DBDIALECT: $DBDIALECT"
  log_proc_msg "LOGS_FILE: $LOGS_FILE"
}

__run_api() {
  log_proc_msg "Running api container"

  sudo docker rm -fv $FAKE_API_CONTAINER_NAME || true
  local run_cmd="sudo docker run \
    -d \
    -e DBNAME=$DBNAME \
    -e DBUSERNAME=$DBUSERNAME \
    -e DBPASSWORD=$DBPASSWORD \
    -e DBHOST=$DBHOST \
    -e DBPORT=$DBPORT \
    -e DBDIALECT=$DBDIALECT \
    --net=host \
    --privileged=true \
    --name=$FAKE_API_CONTAINER_NAME \
    $API_IMAGE
  "

  eval "$run_cmd"
  log_proc_msg "API container started"
}

__check_api() {
  log_proc_msg "Checking API container status"

  local interval=3
  local timeout=180
  local counter=0
  local is_booted=false
  local wait_time_secs=20

  while [ $is_booted != true ] && [ $counter -lt $timeout ]; do
    local running_api_container=$(sudo docker ps | \
      grep $FAKE_API_CONTAINER_NAME | awk '{print $1}')

    if [ "$running_api_container" != "" ]; then
      log_proc_msg "Waiting $wait_time_secs seconds before stopping API container"
      is_booted=true
      sleep $wait_time_secs

      # Check if it's still running
      local api_container=$(sudo docker ps | grep $FAKE_API_CONTAINER_NAME | awk '{print $1}')

      if [ "$api_container" != "" ]; then
        log_proc_msg "Stopping API container"
        sudo docker stop -t=0 $api_container
      fi

      sudo docker rm $running_api_container
    else
      local exited_api_container=$(sudo docker ps -a --filter status=exited | \
        grep $FAKE_API_CONTAINER_NAME | awk '{print $1}')

      if [ "$exited_api_container" != "" ]; then
        log_proc_msg "Removing API container"
        sudo docker rm $exited_api_container
        is_booted=true
      else
        let "counter = $counter + $interval"
        sleep $interval
      fi
    fi
  done

  if [ $is_booted = false ]; then
    log_proc_error "Failed to boot api container"
    exit 1
  fi
}

main() {
  log_proc_marker "Booting fake api to generate models"

  __validate_api_envs
  __run_api
  __check_api

  log_proc_msg "Started API container successfully"
}

main
